"""
Plugin base for Metalsmyth, providing a few conveniences
"""

class Plugin(object):
    """
    A plugin is just a callable that takes two arguments:
        files:
        a dictionary of files, where keys are file paths and values are parsed frontmatter posts.

        stack:
        a Stack instance, for operating on metadata

    This base class provides two methods to make things easier:
        run:
        Override this method to do things. Takes a dictionary of files
        and the current Stack instance. You don't actually have to use
        either of these things.

        __call__:
        A wrapper around `run`, to make an instance callable. You shouldn't
        have to touch this.

        except Exception, e:
            raise e

        __init__:
        As a convenience, the default constructor just saves any
        positional and keyword arguments as `self.args` and `self.kwargs`
        for later use.

    """
    def __init__(self, *args, **kwargs):
        "Stash any init args and kwargs for later, for conveniences"
        self.args = args
        self.kwargs = kwargs

    def __call__(self, files, metalsmyth):
        return self.run(files, metalsmyth)

    def run(self, files, metalsmyth):
        pass


"""
This is a simple drafts plugin that filters out any files marked 'draft' in Frontmatter metadata.
"""

def drafts(files, stack):
    "Filter out any files marked 'draft'"
    for path, post in list(files.items()):
        if post.get('draft'):
            del files[path]



"""
Plugin to parse dates into a proper Python datetime
"""
from dateutil.parser import parse

from . import Plugin


class Dates(Plugin):
    """
    Given a date field name, turn that field into a proper datetime
    """
    
    def __init__(self, date_field='date'):
        self.date_field = date_field
    
    def run(self, files, stack):
        "Convert dates"
        for filename, post in files.items():
            if self.date_field in post.metadata:
                post[self.date_field] = parse(post[self.date_field])


"""
Convert content to html with markdown (and possibly other formats)
"""
from . import Plugin


class Markdown(Plugin):
    """
    Convert markdown content to HTML. Options set in __init__ will be passed to parser.
    """
    def __init__(self, **options):
        # import and initialize here
        import markdown
        self.md = markdown.Markdown(**options)

    def run(self, files, stack):
        "Convert files"
        for filename, post in files.items():
            # reset first to clear any extension state
            post.content = self.md.reset().convert(post.content)


class Bleach(Plugin):
    """
    Clean HTML. Options kwargs set in __init__ will be used with bleach.clean
    """
    def __init__(self, *args, **kwargs):
        # import and stash here to minimize dependencies
        import bleach

        self.bleach = bleach
        self.args = list(args)
        self.kwargs = dict(kwargs)

    def run(self, files, stack):
        "Clean your text"
        for filename, post in files.items():
            post.content = self.bleach.clean(post.content, *self.args, **self.kwargs)


class Linkify(Plugin):
    """
    Run bleach.linkify on post.content
    """
    def __init__(self, *args, **kwargs):
        # again, import here
        import bleach

        self.bleach = bleach
        self.args = list(args)
        self.kwargs = dict(kwargs)
    
    def run(self, files, stack):
        "Linkify your text"
        for filename, post in files.items():
            post.content = self.bleach.linkify(post.content, *self.args, **self.kwargs)





"""
Render posts with templates
"""
import os

from . import Plugin

class Jinja(Plugin):
    """
    Render templates with post as context.
    Use an existing jinja2 environment or simply pass a template directory.
    """
    def __init__(self, template_dir='templates', default_template=None, loader=None, environment=None):
        # do imports here so other template engines can work independently
        from jinja2 import Environment, FileSystemLoader

        # check for environment, then loader, then just build it
        if environment is not None:
            self.env = environment
            self.loader = getattr(environment, 'loader', None)

        elif loader is not None:
            self.env = Environment(loader=loader)
            self.loader = loader

        else:
            self.loader = FileSystemLoader(template_dir)
            self.env = Environment(loader=self.loader)

        if default_template:
            self.default_template = env.get_template(default_template)

    def run(self, files, stack):
        "Render templates"

        for filename, post in files.items():
            # check for a template field
            if "template" in post.metadata:
                template = self.env.get_template(post['template'])

            # or use the default template
            elif hasattr(self, 'default_template'):
                template = self.default_template

            else: # no template, so bail
                continue

            # at this point, we have a template, so render
            post.content = template.render(post=post)



"""
The core of Metalsmyth (this really needs a better name) is a Stack,
which reads files from a source directory, registers middleware
and processes files.
"""
import glob
import os

import frontmatter


class Stack(object):
    """
    A Stack takes a source directory, output directory, optional middleware and metadata
    """
    def __init__(self, source='src', dest='build', *middleware, **metadata):
        self.source = source
        self.dest = dest
        self.middleware = list(middleware)
        self.metadata = dict(metadata)

    def get_files(self):
        """
        Read and parse files from a directory,
        return a dictionary of path => post
        """
        files = {}
        for filename in os.listdir(self.source):
            path = os.path.join(self.source, filename)
            files[filename] = frontmatter.load(path, 
                filename=filename, 
                slug=os.path.basename(filename))

        return files

    def run(self):
        "Run each middleware function on files"
        # load files from source directory
        files = self.get_files()

        # loop through each middleware
        for func in self.middleware:

            # call each one, ignoring return value
            func(files, self)

        # store and return the result
        self.files = files
        return files

    def build(self):
        "Build out results to dest directory (creating if needed)"

        # ensure a build dir
        if not os.path.isdir(self.dest):
            os.makedirs(self.dest)

        # make sure we have files
        files = getattr(self, 'files', self.run())

        # write the content of each post to dest, using keys as filenames
        for filename, post in self.files.items():

            # join filename to dest dir
            path = os.path.join(self.dest, filename)
            with open(path, 'wb') as f:
                f.write(post.content.encode('utf-8'))

    def serialize(self, as_dict=False, sort=None):
        """
        Dump built files as a list or dictionary, for JSON or other serialization.

            sort: a key function to sort a list, or simply True
        """
        files = getattr(self, 'files', self.run())

        if as_dict:
            return dict((fn, p.to_dict()) for fn, p in files.items())

        # generate a list
        data = (p.to_dict() for p in files.values())

        if callable(sort):
            return sorted(data, key=sort)

        elif sort is True:
            return sorted(data)

        return list(data)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.md') as f:
    readme = f.read()


requirements = [
    'python-frontmatter',
    'six'
]


VERSION = '0.1.0'


setup(
    name = 'metalsmyth',
    version = VERSION,
    description = 'Process a directory of files with frontmatter and middleware',
    long_description = readme,
    author = 'Chris Amico',
    author_email = 'eyeseast@gmail.com',
    url = 'https://github.com/eyeseast/python-metalsmyth',
    packages = ['metalsmyth', 'metalsmyth.plugins'],
    include_package_data = True,
    install_requires = requirements,
    license = 'MIT',
    zip_safe = False,
    keywords = 'frontmatter static-generator',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='test',
)
